@@ -1,32 +1,38 @@
-# Copyright (c) 2018 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 from typing import Dict, Optional, List, Set
 
 from PyQt5.QtCore import QObject, pyqtSlot
 
+from UM.Logger import Logger
 from UM.Util import parseBool
 
 from cura.Machines.ContainerNode import ContainerNode
 
 
+##  A QualityGroup represents a group of quality containers that must be applied
+#   to each ContainerStack when it's used.
 #
-# A QualityGroup represents a group of containers that must be applied to each ContainerStack when it's used.
-# Some concrete examples are Quality and QualityChanges: when we select quality type "normal", this quality type
-# must be applied to all stacks in a machine, although each stack can have different containers. Use an Ultimaker 3
-# as an example, suppose we choose quality type "normal", the actual InstanceContainers on each stack may look
-# as below:
-#                       GlobalStack         ExtruderStack 1         ExtruderStack 2
-# quality container:    um3_global_normal   um3_aa04_pla_normal     um3_aa04_abs_normal
+#   A concrete example: When there are two extruders and the user selects the
+#   quality type "normal", this quality type must be applied to all stacks in a
+#   machine, although each stack can have different containers. So one global
+#   profile gets put on the global stack and one extruder profile gets put on
+#   each extruder stack. This quality group then contains the following
+#   profiles (for instance):
+#                      GlobalStack       ExtruderStack 1     ExtruderStack 2
+#   quality container: um3_global_normal um3_aa04_pla_normal um3_aa04_abs_normal
 #
-# This QualityGroup is mainly used in quality and quality_changes to group the containers that can be applied to
-# a machine, so when a quality/custom quality is selected, the container can be directly applied to each stack instead
-# of looking them up again.
-#
-class QualityGroup(QObject):
-
-    def __init__(self, name: str, quality_type: str, parent = None) -> None:
-        super().__init__(parent)
+#   The purpose of these quality groups is to group the containers that can be
+#   applied to a configuration, so that when a quality level is selected, the
+#   container can directly be applied to each stack instead of looking them up
+#   again.
+class QualityGroup:
+    ##  Constructs a new group.
+    #   \param name The user-visible name for the group.
+    #   \param quality_type The quality level that each profile in this group
+    #   has.
+    def __init__(self, name: str, quality_type: str) -> None:
         self.name = name
         self.node_for_global = None  # type: Optional[ContainerNode]
         self.nodes_for_extruders = {}  # type: Dict[int, ContainerNode]
@@ -34,7 +40,6 @@ class QualityGroup(QObject):
         self.is_available = False
         self.is_experimental = False
 
-    @pyqtSlot(result = str)
     def getName(self) -> str:
         return self.name
 
@@ -43,7 +48,7 @@ class QualityGroup(QObject):
         for node in [self.node_for_global] + list(self.nodes_for_extruders.values()):
             if node is None:
                 continue
-            container = node.getContainer()
+            container = node.container
             if container:
                 result.update(container.getAllKeys())
         return result
@@ -60,6 +65,9 @@ class QualityGroup(QObject):
         self.node_for_global = node
 
         # Update is_experimental flag
+        if not node.container:
+            Logger.log("w", "Node {0} doesn't have a container.".format(node.container_id))
+            return
         is_experimental = parseBool(node.getMetaDataEntry("is_experimental", False))
         self.is_experimental |= is_experimental
 
@@ -67,5 +75,8 @@ class QualityGroup(QObject):
         self.nodes_for_extruders[position] = node
 
         # Update is_experimental flag
+        if not node.container:
+            Logger.log("w", "Node {0} doesn't have a container.".format(node.container_id))
+            return
         is_experimental = parseBool(node.getMetaDataEntry("is_experimental", False))
         self.is_experimental |= is_experimental
