@@ -2,10 +2,12 @@
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import copy
+from typing import List
 
 from UM.Job import Job
 from UM.Operations.GroupedOperation import GroupedOperation
 from UM.Message import Message
+from UM.Scene.SceneNode import SceneNode
 from UM.i18n import i18nCatalog
 i18n_catalog = i18nCatalog("cura")
 
@@ -23,7 +25,7 @@ class MultiplyObjectsJob(Job):
         self._count = count
         self._min_offset = min_offset
 
-    def run(self):
+    def run(self) -> None:
         status_message = Message(i18n_catalog.i18nc("@info:status", "Multiplying and placing objects"), lifetime=0,
                                  dismissable=False, progress=0, title = i18n_catalog.i18nc("@info:title", "Placing Objects"))
         status_message.show()
@@ -33,17 +35,19 @@ class MultiplyObjectsJob(Job):
         current_progress = 0
 
         global_container_stack = Application.getInstance().getGlobalContainerStack()
+        if global_container_stack is None:
+            return  # We can't do anything in this case.
         machine_width = global_container_stack.getProperty("machine_width", "value")
         machine_depth = global_container_stack.getProperty("machine_depth", "value")
 
         root = scene.getRoot()
         scale = 0.5
         arranger = Arrange.create(x = machine_width, y = machine_depth, scene_root = root, scale = scale, min_offset = self._min_offset)
-        processed_nodes = []
+        processed_nodes = []  # type: List[SceneNode]
         nodes = []
 
         not_fit_count = 0
-
+        found_solution_for_all = False
         for node in self._objects:
             # If object is part of a group, multiply group
             current_node = node
@@ -62,12 +66,16 @@ class MultiplyObjectsJob(Job):
 
             found_solution_for_all = True
             arranger.resetLastPriority()
-            for i in range(self._count):
+            for _ in range(self._count):
                 # We do place the nodes one by one, as we want to yield in between.
                 new_node = copy.deepcopy(node)
                 solution_found = False
                 if not node_too_big:
-                    solution_found = arranger.findNodePlacement(new_node, offset_shape_arr, hull_shape_arr)
+                    if offset_shape_arr is not None and hull_shape_arr is not None:
+                        solution_found = arranger.findNodePlacement(new_node, offset_shape_arr, hull_shape_arr)
+                    else:
+                        # The node has no shape, so no need to arrange it. The solution is simple: Do nothing. 
+                        solution_found = True
 
                 if node_too_big or not solution_found:
                     found_solution_for_all = False
@@ -90,10 +98,10 @@ class MultiplyObjectsJob(Job):
             Job.yieldThread()
 
         if nodes:
-            op = GroupedOperation()
+            operation = GroupedOperation()
             for new_node in nodes:
-                op.addOperation(AddSceneNodeOperation(new_node, current_node.getParent()))
-            op.push()
+                operation.addOperation(AddSceneNodeOperation(new_node, current_node.getParent()))
+            operation.push()
         status_message.hide()
 
         if not found_solution_for_all:
