@@ -1,7 +1,8 @@
 # Copyright (c) 2018 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
-from typing import Optional, TYPE_CHECKING
+from typing import Optional, TYPE_CHECKING, cast
+
 
 from UM.Application import Application
 from UM.Resources import Resources
@@ -12,12 +13,10 @@ from UM.View.RenderBatch import RenderBatch
 
 
 from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator
+from cura.Scene.CuraSceneNode import CuraSceneNode
 
 if TYPE_CHECKING:
     from UM.View.GL.ShaderProgram import ShaderProgram
-
-MYPY = False
-if MYPY:
     from UM.Scene.Camera import Camera
 
 
@@ -44,9 +43,9 @@ class PreviewPass(RenderPass):
 
         self._renderer = Application.getInstance().getRenderer()
 
-        self._shader = None #type: Optional[ShaderProgram]
-        self._non_printing_shader = None #type: Optional[ShaderProgram]
-        self._support_mesh_shader = None #type: Optional[ShaderProgram]
+        self._shader = None  # type: Optional[ShaderProgram]
+        self._non_printing_shader = None  # type: Optional[ShaderProgram]
+        self._support_mesh_shader = None  # type: Optional[ShaderProgram]
         self._scene = Application.getInstance().getController().getScene()
 
     #   Set the camera to be used by this render pass
@@ -62,6 +61,7 @@ class PreviewPass(RenderPass):
                 self._shader.setUniformValue("u_ambientColor", [0.1, 0.1, 0.1, 1.0])
                 self._shader.setUniformValue("u_specularColor", [0.6, 0.6, 0.6, 1.0])
                 self._shader.setUniformValue("u_shininess", 20.0)
+                self._shader.setUniformValue("u_faceId", -1)  # Don't render any selected faces in the preview.
 
         if not self._non_printing_shader:
             if self._non_printing_shader:
@@ -83,8 +83,8 @@ class PreviewPass(RenderPass):
         batch_support_mesh = RenderBatch(self._support_mesh_shader)
 
         # Fill up the batch with objects that can be sliced.
-        for node in DepthFirstIterator(self._scene.getRoot()): #type: ignore #Ignore type error because iter() should get called automatically by Python syntax.
-            if hasattr(node, "_outside_buildarea") and not node._outside_buildarea:
+        for node in DepthFirstIterator(self._scene.getRoot()):
+            if hasattr(node, "_outside_buildarea") and not getattr(node, "_outside_buildarea"):
                 if node.callDecoration("isSliceable") and node.getMeshData() and node.isVisible():
                     per_mesh_stack = node.callDecoration("getStack")
                     if node.callDecoration("isNonThumbnailVisibleMesh"):
@@ -94,7 +94,7 @@ class PreviewPass(RenderPass):
                         # Support mesh
                         uniforms = {}
                         shade_factor = 0.6
-                        diffuse_color = node.getDiffuseColor()
+                        diffuse_color = cast(CuraSceneNode, node).getDiffuseColor()
                         diffuse_color2 = [
                             diffuse_color[0] * shade_factor,
                             diffuse_color[1] * shade_factor,
@@ -106,7 +106,7 @@ class PreviewPass(RenderPass):
                     else:
                         # Normal scene node
                         uniforms = {}
-                        uniforms["diffuse_color"] = prettier_color(node.getDiffuseColor())
+                        uniforms["diffuse_color"] = prettier_color(cast(CuraSceneNode, node).getDiffuseColor())
                         batch.addItem(node.getWorldTransformation(), node.getMeshData(), uniforms = uniforms)
 
         self.bind()
